// -----------------------------------------------------------------------
//
//   Copyright 2004 H. Peter Anvin - All Rights Reserved
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, Inc., 53 Temple Place Ste 330,
//   Bostom MA 02111-1307, USA; either version 2 of the License, or
//   (at your option) any later version; incorporated herein by reference.
//
// -----------------------------------------------------------------------
// $Id: printer.v,v 1.2 2004/09/16 01:08:52 hpa Exp $

//
// printer.v
//
// Simple unidirectional serial printer driver
//

module printer (
	       reset_n,		// Global reset
	       clk,		// Clock (25 MHz)
	       
	       tty_txd,		// Serial port data out
	       tty_cts,		// Serial port flow control in

	       abc_cs_n,	// ABC-bus CS#
	       abc_out_n,	// ABC-bus OUT#
	       abc_c1_n,	// ABC-bus C1#
	       abc_c2_n,	// ABC-bus C2#
	       abc_c3_n,	// ABC-bus C3#
	       abc_c4_n,	// ABC-bus C4#
	       abc_inp_n,	// ABC-bus INP#
	       abc_status_n,	// ABC-bus STATUS#
	       abc_rst_n,       // ABC-bus RST#

	       abc_do,		// ABC-bus data out (cpu->card)
	       abc_di,		// ABC-bus data in  (card->cpu)

	       select		// Select LED
	       );

   input        reset_n;
   input        clk;

   output       tty_txd;
   input        tty_cts;
   
   input        abc_cs_n;
   input        abc_out_n;
   input        abc_c1_n;
   input        abc_c2_n;
   input        abc_c3_n;
   input        abc_c4_n;
   input        abc_inp_n;
   input        abc_status_n;
   input        abc_rst_n;

   input [7:0]  abc_do;
   output [7:0] abc_di;
   
   output 	select;

   // Which select code this device uses
   parameter     selectcode = 6'd24;
   reg           selected;
   assign 	 select = selected; // LED

   // Data out
   reg [8:0] 	 data_sr;	// Serial port shift register
   reg 		 data_out;	// Serial port data out
   assign        tty_txd = data_out;
   
   // Baud rate divider
   reg [15:0] 	 baud_rate;
   reg [15:0] 	 baud_rate_ctr;
   // Time to advance the serial register
   wire 	 advance_bit = (baud_rate_ctr == 0);

   // BREAK generation counter
   reg [8:0] 	 break_ctr;
   wire 	 break_bit = break_ctr[8];

   // BUSY counter
   reg [4:0] 	 busy_ctr;
   wire 	 busy_bit = ~busy_ctr[4];
   
   // Status generation
   wire [7:0] 	 status = { 5'b0, ~tty_cts, ~break_bit, busy_bit };
   assign 	 abc_di = (selected & ~abc_status_n) ? status : 8'hFF;
      
   // Edge detect
   reg 		 abc_out_q;
   reg 		 abc_c1_q;
      
   always @(negedge reset_n or posedge clk)
     begin
	if ( ~reset_n )
	  begin
	     selected      <= 0;
	     baud_rate     <= 216; // 115200 bps
	     baud_rate_ctr <= 0;
	     data_sr       <= ~9'b0;
	     data_out      <= 1;
	     break_ctr     <= ~9'b0;
	     busy_ctr      <= ~5'b0;
	     abc_out_q     <= 1;
	     abc_c1_q      <= 1;
	  end
	else
	  begin

	     if ( ~abc_rst_n )
               selected <= 0;
	     else if ( ~abc_cs_n )
	       selected <= (abc_do[5:0] == selectcode);

	     if ( selected & ~abc_out_n & abc_out_q )
	       data_sr <= { abc_do, 1'b0 };
	     else if ( advance_bit )
	       data_sr <= { break_bit, data_sr[8:1] };

	     if ( advance_bit )
	       data_out <= data_sr[0];

	     if ( advance_bit )
	       baud_rate_ctr <= baud_rate;
	     else
	       baud_rate_ctr <= baud_rate_ctr - 1;

	     if ( selected & ~abc_c1_n & abc_c1_q )
	       break_ctr <= { 1'b0, abc_do };
	     else if ( advance_bit & ~break_bit )
	       break_ctr <= break_ctr - 1;

	     if ( selected & ~abc_out_n & abc_out_q )
	       busy_ctr <= 5'd9; // Note that the high bit is zero
	     else if ( advance_bit & busy_bit )
	       busy_ctr <= busy_ctr - 1;

	     if ( selected & ~abc_c2_n )
	       baud_rate[7:0] <= abc_do;

	     if ( selected & ~abc_c3_n )
	       baud_rate[15:8] <= abc_do;

	     abc_out_q <= abc_out_n;
	     abc_c1_q  <= abc_c1_n;
	     
	  end // else: !if( ~reset_n )
     end // always @ (negedge reset_n or posedge clk)

endmodule // printer

	     
	     
   

   

	       
	       
